<?php

namespace Slack\Blockkit\Blocks;

use Slack\Blockkit\Blocks;
use Slack\Blockkit\Blocks\Elements\Text;
use Slack\Exceptions\SlackSyntaxException;

final class Header extends Blocks
{
	protected const LIMITS = [
		'block_id' => 255,			// @todo Should be unique for each message
		'text' => 150,
	];

	/**
	 * @param Text $text
	 * @throws SlackSyntaxException
	 */
	public function __construct(Text $text)
	{
		parent::__construct();

		// Defaults
		$this->type = 'header';

		if ($text->type != 'plain_text')
			throw new SlackSyntaxException(sprintf('Text must be plain_text not %s',$text->type));

		if (strlen($text->text) > self::LIMITS['text'])
			throw new SlackSyntaxException(sprintf('Text must be %d chars or less',self::LIMITS['text']));

		$this->text = $text;
	}

	public static function item(Text $text): self
	{
		return new self($text);
	}

	/* OPTIONAL ITEMS */

	public function block_id(string $string): self
	{
		$this->block_id = $this->validate('block_id',$string);

		return $this;
	}
}